@@ -110,6 +110,8 @@ To see it in action, see this
 import os
 from itertools import chain
 
+from tf.core.helpers import specFromRangesLogical, specFromRanges, rangesFromSet
+
 ZWJ = "\u200d"  # zero width joiner
 
 
@@ -202,8 +204,8 @@ class Recorder:
         """
         return "".join(self.material)
 
-    def positions(self, byType=False):
-        """Get the node positions.
+    def positions(self, byType=False, simple=False):
+        """Get the node positions as mapping from character positions.
 
         Parameters
         ----------
@@ -211,22 +213,31 @@ class Recorder:
             If True, makes a separate node mapping per node type.
             For this it is needed that the Recorder has been
             passed a TF api when it was initialized.
+        simple: boolean, optional `False`
+            In some cases it is known on beforehand that at each textual position
+            there is at most 1 node.
+            Then it is more economical to fill the list with single nodes
+            rather than with sets of nodes.
+            If this parameter is True, we pick the first node from the set.
 
         Returns
         -------
         list|dict|None
             If `byType`, the result is a dictionary, keyed by node type,
-            with values the mapping for nodes of that type.
-            Entry `i` in these list contains the frozen set of all
-            nodes of that type that were active at character position `i` in the text.
+            with values the mapping of textual positions to nodes of that type.
+            This mapping takes the shape of a list where entry `i`
+            contains the frozen set of all nodes of that type
+            that were active at character position `i` in the text.
 
-            If not `byType` then a single mapping list is returned, where
-            entry `i` contains the frozen set of all
+            If not `byType` then a single mapping is returned (as list),
+            where entry `i` contains the frozen set of all
             nodes, irrespective of their type
             that were active at character position `i` in the text.
         """
 
         if not byType:
+            if simple:
+                return tuple(list(x)[0] if x else None for x in self.nodesByPos)
             return self.nodesByPos
 
         api = self.api
@@ -268,12 +279,218 @@ where `api` is the result of
                 thisSet = (
                     frozenset(typed[nodeType]) if nodeType in typed else frozenset()
                 )
-                nodesByPosByType[nodeType].append(thisSet)
+                value = (list(thisSet)[0] if thisSet else None) if simple else thisSet
+                nodesByPosByType[nodeType].append(value)
 
         info("done")
         indent(level=False)
         return nodesByPosByType
 
+    def iPositions(self, byType=False, logical=True, asEntries=False):
+        """Get the node positions as mapping from nodes.
+
+        Parameters
+        ----------
+        byType: boolean, optional `False`
+            If True, makes a separate node mapping per node type.
+            For this it is needed that the Recorder has been
+            passed a TF api when it was initialized.
+        logical: boolean, optional `True`
+            If True, specs are represented as tuples of ranges
+            and a range is represented as a tuples of a begin and end point,
+            or as a tuple of a single point.
+            Points are integers.
+            If False, ranges are represented by strings: , separated ranges,
+            a ranges is b-e or p.
+        asEntries: boolean, optional `False`
+            If True, do not return the dict, but rather its entries.
+
+        Returns
+        -------
+        list|dict|None
+            If `byType`, the result is a dictionary, keyed by node type,
+            with values the mapping for nodes of that type.
+            Entry `n` in this mapping contains the intervals of all
+            character positions in the text where node `n` is active.
+
+            If not `byType` then a single mapping is returned.
+        """
+
+        method = specFromRangesLogical if logical else specFromRanges
+        posByNode = {}
+        for (i, nodeSet) in enumerate(self.nodesByPos):
+            for node in nodeSet:
+                posByNode.setdefault(node, set()).add(i)
+        for (n, nodeSet) in posByNode.items():
+            posByNode[n] = method(rangesFromSet(nodeSet))
+
+        if asEntries:
+            posByNode = tuple(posByNode.items())
+        if not byType:
+            return posByNode
+
+        api = self.api
+        if api is None:
+            print(
+                """\
+Cannot determine node types without a TF api.
+You have to call Recorder(`api`) instead of Recorder()
+where `api` is the result of
+    tf.app.use(corpus)
+    or
+    tf.Fabric(locations, modules).load(features)
+"""
+            )
+            return None
+
+        F = api.F
+        Fotypev = F.otype.v
+
+        posByNodeType = {}
+        if asEntries:
+            for (n, spec) in posByNode:
+                nType = Fotypev(n)
+                posByNodeType.setdefault(nType, []).append(n, spec)
+        else:
+            for (n, spec) in posByNode.items():
+                nType = Fotypev(n)
+                posByNodeType.setdefault(nType, {})[n] = spec
+
+        return posByNodeType
+
+    def rPositions(self, acceptMaterialOutsideNodes=False):
+        """Get the first textual position for each node
+
+        The position information is a big amount of data, in the general case.
+        Under certain assumptions we can economize on this data usage.
+
+        Strong assumptions:
+
+        1.  every textual position is covered by exactly one node**
+        2.  the nodes are consecutive:
+            every next node is equal to the previous node plus 1
+        3.  the positions of the nodes are monotonous in the nodes, i.e.
+            if node n < m, then the position of n is before the position of m.
+
+        Imagine the text partitioned in consecutive non-overlapping chunks, where
+        each node corresponds to exactly one chunk, and the order of the nodes
+        is the same as the order of the corresponding chunks.
+
+        Parameters
+        ----------
+        acceptMaterialOutsideNodes: boolean, optional `False`
+            If this is True, we accept that the text contains extra material that is not
+            covered by any node.
+            So condition 1 above is relaxed in that we accept no nodes for a textual
+            position.
+
+            Applications that make use of the positions must realize that in this case
+            the material associated with a node also includes the subsequent material
+            outside any node.
+
+        Returns
+        -------
+        list | str
+            The result is a list `posList`.
+            Suppose we map nodes n, n+1, ..., n+m.
+            Suppose node n starts at position t0, n+1 at position t1,
+            n+m at position tm.
+            Suppose te is the position just after the whole text.
+
+            Then we deliver as a result:
+
+            ```
+            n - 1
+            t0
+            t1
+            ...
+            tm
+            te
+            ```
+
+            So the first element of the list is used to specify the offset to be
+            applied for all subsequent nodes.
+            The `te` value is added as a sentinel, to facilitate the determination
+            of the last position of each node.
+
+            Users of `posList` can find the start and end positions of node m
+            as follows
+
+            ```
+            start = posList[m - posList[0]]
+            end   = posList[m - posList[0] + 1] - 1
+            ```
+
+            We return not only the posList if the assumptions hold.
+            If not, we return a string with diagnostic information.
+        """
+
+        good = True
+        multipleNodes = 0
+        multipleFirst = 0
+        noNodes = 0
+        noFirst = 0
+        nonConsecutive = 0
+        nonConsecutiveFirst = 0
+
+        posByNode = {}
+        for (i, nodeSet) in enumerate(self.nodesByPos):
+            if (not acceptMaterialOutsideNodes and len(nodeSet) == 0) or len(
+                nodeSet
+            ) > 1:
+                good = False
+                if len(nodeSet) == 0:
+                    if noNodes == 0:
+                        noFirst = i
+                    noNodes += 1
+                else:
+                    if multipleNodes == 0:
+                        multipleFirst = i
+                    multipleNodes += 1
+                continue
+            for node in nodeSet:
+                if node in posByNode:
+                    continue
+                posByNode[node] = i
+
+        lastI = i
+
+        if not good:
+            msg = ""
+            if noNodes:
+                msg += (
+                    f"{noNodes} positions without node, "
+                    f"of which the first one is {noFirst}\n"
+                )
+            if multipleNodes:
+                msg += (
+                    f"{multipleNodes} positions with multiple nodes, "
+                    f"of which the first one is {multipleFirst}\n"
+                )
+            return msg
+
+        sortedPosByNode = sorted(posByNode.items())
+        offset = sortedPosByNode[0][0] - 1
+        posList = [offset]
+        prevNode = offset
+        for (node, i) in sortedPosByNode:
+            if prevNode + 1 != node:
+                good = False
+                if nonConsecutive == 0:
+                    nonConsecutiveFirst = f"{prevNode} => {node}"
+                nonConsecutive += 1
+            else:
+                posList.append(i)
+            prevNode = node
+        posList.append(lastI)
+
+        if not good:
+            return (
+                f"{nonConsecutive} nonConsecutive nodes, "
+                f"of which the first one is {nonConsecutiveFirst}"
+            )
+        return posList
+
     def write(self, textPath, posPath=None, byType=False, optimize=True):
         """Write the recorder information to disk.
 
