@@ -1,5 +1,9 @@
 """
-.. include:: ../../docs/core/oslotsfeature.md
+# Access to *oslots* feature data.
+
+In general, features are stored as dictionaries, but this specific feature
+has an optimized representation. Since it is a large feature and present
+in any TF dataset, this pays off.
 """
 
 
