@@ -1,4 +1,5 @@
-"""Functions that precompute data.
+"""
+# Precompute data.
 
 For Text-Fabric to work efficiently, some  derived data needs to be precomputed.
 The precomputed data has a similar function as indexes in a database.
@@ -614,7 +615,7 @@ def structure(info, error, otype, oslots, otext, rank, levUp, *sFeats):
 
     sTypes = set(sTypeList)
     if len(sTypes) != nsTypes:
-        error(f"WARNING: duplicate structure levels")
+        error("WARNING: duplicate structure levels")
         return ({}, {})
 
     higherTypes = collections.defaultdict(set)
