@@ -1,3 +1,15 @@
+"""
+# Dress TF nodes up for serving on the web
+
+When the TF kernel has retrieved data, it comes in the form of nodes.
+
+But the kernel is the one that is able to dress those nodes up with
+meaningful data.
+
+That dressing up is happening in this module, it has the higher level
+functions for composing tables and passages.
+"""
+
 import pickle
 import markdown
 from flask import jsonify, redirect, render_template, make_response
@@ -236,7 +248,8 @@ def serveDownload(web):
     condenseType = form["condenseType"] or None
     textFormat = form["textFormat"] or None
     csvs = None
-    resultsX = None
+    tupleResultsX = None
+    queryResultsX = None
     messages = ""
     if task in wildQueries:
         messages = (
@@ -245,7 +258,7 @@ def serveDownload(web):
         )
     else:
         try:
-            (queryMessages, csvs, resultsX) = kernelApi.csvs(
+            (queryMessages, csvs, tupleResultsX, queryResultsX) = kernelApi.csvs(
                 task,
                 form["tuples"],
                 form["sections"],
@@ -271,9 +284,10 @@ def serveDownload(web):
     (provenanceHtml, provenanceMd) = wrapProvenance(form, provenance, setNames)
 
     csvs = pickle.loads(csvs)
-    resultsX = pickle.loads(resultsX)
+    tupleResultsX = pickle.loads(tupleResultsX)
+    queryResultsX = pickle.loads(queryResultsX)
     about = getAbout(header, provenanceMd, form)
-    (fileName, zipBuffer) = zipData(csvs, resultsX, about, form)
+    (fileName, zipBuffer) = zipData(csvs, tupleResultsX, queryResultsX, about, form)
 
     headers = {
         "Expires": "0",
