@@ -1,6 +1,48 @@
 """
-.. include:: ../../docs/writing/transcription.md
+# Transcription
+
+Text-Fabric has support for several writing systems, by means of
+transcription tables and fonts that will be invoked when displaying the main text.
+
+It also calls functions to use these tables for converting Hebrew and Syriac
+text material to transliterated representations and back.
+
+There is also a phonetic transcription for Hebrew, designed in
+[phono.ipynb](https://nbviewer.jupyter.org/github/etcbc/phono/blob/master/programs/phono.ipynb)
+
+## Character tables and fonts
+
+### `hbo` Hebrew
+
+`tf.writing.hebrew`: full list of characters covered by the ETCBC and phonetic transcriptions
+
+Font `Ezra SIL`.
+
+### `syc` Syriac
+
+`tf.writing.syriac`: full list of characters covered by the ETCBC transcriptions
+
+Font `Estrangelo Edessa`.
+
+### `ara` Arabic
+
+`tf.writing.arabic`: full list of characters covered by the transcription used for the Quran
+
+Font `AmiriQuran`.
+
+### `grc` Greek
+
+Font `Gentium`.
+
+### `akk` Akkadian
+
+Font `Santakku`.
+
+### `cld` Neo Aramaic
+
+Font `CharisSIL-R`.
 """
+
 import re
 
 
@@ -419,6 +461,7 @@ class Transcription(object):
     }
     """
     Maps an Arabic transliteration character to Unicode.
+    This is the mapping used in the Quran representation on tanzil.net.
 
     Example: beh
 
@@ -439,7 +482,126 @@ class Transcription(object):
         b
     """
 
-    arabic_mappingi = dict((v, k) for (k, v) in arabic_mapping.items())
+    arabic_mappingi = {v: k for (k, v) in arabic_mapping.items()}
+
+    arabicTrans = {
+        "\u0020": (" ", " ", " ", " "),      # SPACE
+        "\u060c": ("‚", ",", ",", ","),      # ARABIC COMMA
+        "\u061b": ("„", ";", ";", ";"),      # ARABIC SEMICOLON
+        "\u061f": ("?", "?", "?", "?"),      # ARABIC QUESTION MARK
+        "\u0621": ("'", "'", "ʾ", "'"),      # ARABIC LETTER HAMZA
+        "\u0622": ("»", "_a", "ā", "ā"),     # ARABIC LETTER ALEF WITH MADDA ABOVE
+        "\u0623": (">", "_a", "ā", "ā"),     # ARABIC LETTER ALEF WITH HAMZA ABOVE
+        "\u0624": ("&", "'", "ʾ", "'"),      # ARABIC LETTER WAW WITH HAMZA ABOVE
+        "\u0625": ("<", "_a", "ā", "ā"),     # ARABIC LETTER ALEF WITH HAMZA BELOW
+        "\u0626": ("}", "'", "ʾ", "y"),      # ARABIC LETTER YEH WITH HAMZA ABOVE
+        "\u0627": ("A", "_a", "ā", "ā"),     # ARABIC LETTER ALEF
+        "\u0628": ("b", "b", "b", "b"),      # ARABIC LETTER BEH
+        "\u0629": ("p", "=t", "ŧ", "t"),     # ARABIC LETTER TEH MARBUTA
+        "\u062a": ("t", "t", "t", "t"),      # ARABIC LETTER TEH
+        "\u062b": ("v", "_t", "ṯ", "th"),    # ARABIC LETTER THEH
+        "\u062c": ("j", "j", "ǧ", "j"),      # ARABIC LETTER JEEM
+        "\u062d": ("H", "*h", "ḥ", "ḥ"),     # ARABIC LETTER HAH
+        "\u062e": ("x", "_h", "ḫ", "kh"),    # ARABIC LETTER KHAH
+        "\u062f": ("d", "d", "d", "d"),      # ARABIC LETTER DAL
+        "\u0630": ("*", "_d", "ḏ", "dh"),    # ARABIC LETTER THAL
+        "\u0631": ("r", "r", "r", "r"),      # ARABIC LETTER REH
+        "\u0632": ("z", "z", "z", "z"),      # ARABIC LETTER ZAIN
+        "\u0633": ("s", "s", "s", "s"),      # ARABIC LETTER SEEN
+        "\u0634": ("$", "^s", "š", "sh"),    # ARABIC LETTER SHEEN
+        "\u0635": ("S", "*s", "ṣ", "ṣ"),     # ARABIC LETTER SAD
+        "\u0636": ("D", "*d", "ḍ", "ḍ"),     # ARABIC LETTER DAD
+        "\u0637": ("T", "*t", "ṭ", "ṭ"),     # ARABIC LETTER TAH
+        "\u0638": ("Z", "*z", "ẓ", "ẓ"),     # ARABIC LETTER ZAH
+        "\u0639": ("E", "`", "ʿ", "`"),      # ARABIC LETTER AIN
+        "\u063a": ("g", "*g", "ġ", "gh"),    # ARABIC LETTER GHAIN
+        "\u0640": ("_", "", "", ""),         # ARABIC TATWEEL
+        "\u0641": ("f", "f", "f", "f"),      # ARABIC LETTER FEH
+        "\u0642": ("q", "*k", "ḳ", "q"),     # ARABIC LETTER QAF
+        "\u0643": ("k", "k", "k", "k"),      # ARABIC LETTER KAF
+        "\u0644": ("l", "l", "l", "l"),      # ARABIC LETTER LAM
+        "\u0645": ("m", "m", "m", "m"),      # ARABIC LETTER MEEM
+        "\u0646": ("n", "n", "n", "n"),      # ARABIC LETTER NOON
+        "\u0647": ("h", "h", "h", "h"),      # ARABIC LETTER HEH
+        "\u0648": ("w", "w", "w", "w"),      # ARABIC LETTER WAW
+        "\u0649": ("Y", "/a", "á", "ā"),     # ARABIC LETTER ALEF MAKSURA
+        "\u064a": ("y", "y", "y", "y"),      # ARABIC LETTER YEH
+        "\u064b": ("F", "a*n", "aⁿ", "an"),  # ARABIC FATHATAN
+        "\u064c": ("N", "u*n", "uⁿ", "un"),  # ARABIC DAMMATAN
+        "\u064d": ("K", "i*n", "iⁿ", "in"),  # ARABIC KASRATAN
+        "\u064e": ("a", "a", "a", "a"),      # ARABIC FATHA
+        "\u064f": ("u", "u", "u", "u"),      # ARABIC DAMMA
+        "\u0650": ("i", "i", "i", "i"),      # ARABIC KASRA
+        "\u0651": ("~", "u", "u", "ūw"),     # ARABIC SHADDA
+        "\u0652": ("o", "a", "a", "a"),      # ARABIC SUKUN
+        "\u0653": ("^", "_a", "ā", "ā"),     # ARABIC MADDAH ABOVE
+        "\u0654": ("#", "'", "ʾ", "ā"),      # ARABIC HAMZA ABOVE
+        "\u0655": ("=", "'", "ʾ", "ā"),      # ARABIC HAMZA BELOW
+        "\u0660": ("0", "0", "0", "0"),      # ARABIC INDIC DIGIT ZERO
+        "\u0661": ("1", "1", "1", "1"),      # ARABIC INDIC DIGIT ONE
+        "\u0662": ("2", "2", "2", "2"),      # ARABIC INDIC DIGIT TWO
+        "\u0663": ("3", "3", "3", "3"),      # ARABIC INDIC DIGIT THREE
+        "\u0664": ("4", "4", "4", "4"),      # ARABIC INDIC DIGIT FOUR
+        "\u0665": ("5", "5", "5", "5"),      # ARABIC INDIC DIGIT FIVE
+        "\u0666": ("6", "6", "6", "6"),      # ARABIC INDIC DIGIT SIX
+        "\u0667": ("7", "7", "7", "7"),      # ARABIC INDIC DIGIT SEVEN
+        "\u0668": ("8", "8", "8", "8"),      # ARABIC INDIC DIGIT EIGHT
+        "\u0669": ("9", "9", "9", "9"),      # ARABIC INDIC DIGIT NINE
+        "\u0670": ("`", "~a", "ã", ""),      # ARABIC LETTER SUPERSCRIPT ALEF
+        "\u0671": ("{", "a", "a", "a"),      # ARABIC LETTER ALEF WASLA
+        "\u06af": ("G", "g", "g", "g"),      # ARABIC LETTER GAF
+        "\u06cc": ("J", "#y", "Y", "y"),     # ARABIC LETTER FARSI YEH
+        "\u06d6": ("SlY", "*sl/a", "ṣlá", "ṣla"),  # ARABIC SMALL HIGH LIGATURE SAD WITH LAM WITH ALEF MAKSURA
+        "\u06da": ("M", "#j", "J", "j"),     # ARABIC SMALL HIGH JEEM
+        "\u06dc": (":", "#s", "S", "s"),     # ARABIC SMALL HIGH SEEN
+        "\u06df": ("@", "0", "0", "0"),      # ARABIC SMALL HIGH ROUNDED ZERO
+        "\u06e0": ('"', "0", "0", "0"),      # ARABIC SMALL HIGH UPRIGHT RECTANGULAR ZERO
+        "\u06e2": ("[", "#m", "M", "M"),     # ARABIC SMALL HIGH MEEM ISOLATED FORM
+        "\u06e3": (";", "#s", "S", "S"),     # ARABIC SMALL LOW SEEN
+        "\u06e5": (",", "#w", "W", "W"),     # ARABIC SMALL WAW
+        "\u06e6": (".", "#y", "Y", "Y"),     # ARABIC SMALL YEH
+        "\u06e8": ("!", "#n", "N", "N"),     # ARABIC SMALL HIGH NOON
+        "\u06ea": ("-", ".", ".", "."),      # ARABIC EMPTY CENTRE LOW STOP
+        "\u06eb": ("+", ".", ".", "."),      # ARABIC EMPTY CENTRE HIGH STOP
+        "\u06ec": ("%", ".", ".", "."),      # ARABIC ROUNDED HIGH STOP WITH FILLED CENTRE
+        "\u06ed": ("]", "#m", "M", "M"),     # ARABIC SMALL LOW MEEM
+    }
+    """
+    More Arabic transcriptions:
+
+    * column 1: `custom` [Quran-tanzil](http://tanzil.net/#1:1), slightly extended
+    * column 2/: `ascii` resp. `latin` plus diacritics also known as betacode.
+      We use a list compiled by
+      [Peter Verkinderen](https://pverkind.github.io/betacodeTranscriber/js/betacode.js)
+    * column 4: `standard` (Library of Congress) (to-be filled).
+      We use the
+      [arabic romanization list of 2012](https://www.loc.gov/catdir/cpso/romanization/arabic.pdf)
+      We refrain of from applying rules that cannot be computed without
+      lexical/grammatical/dialectical knowledge of the arabic language.
+    """
+
+    arabicTransQuran = {}
+    arabicTransAscii = {}
+    arabicTransLatin = {}
+    arabicTransStandard = {}
+
+    for (ara, (qur, asc, lat, std)) in arabicTrans.items():
+        arabicTransQuran[ara] = qur
+        arabicTransAscii[ara] = asc
+        arabicTransLatin[ara] = lat
+        arabicTransStandard[ara] = std
+
+    def quranFromArabic(word):
+        return "".join(Transcription.arabicTransQuran.get(x, x) for x in word)
+
+    def asciiFromArabic(word):
+        return "".join(Transcription.arabicTransAscii.get(x, x) for x in word)
+
+    def latinFromArabic(word):
+        return "".join(Transcription.arabicTransLatin.get(x, x) for x in word)
+
+    def standardFromArabic(word):
+        return "".join(Transcription.arabicTransStandard.get(x, x) for x in word)
 
     def __init__(self):
         self.hebrew_consonants = {
@@ -471,6 +633,18 @@ class Transcription(object):
             Transcription.syriac_mapping[c] for c in self.syriac_punctuation_trans
         )
 
+        arabicMap = dict(qur={}, asc={}, lat={}, std={})
+        arabicMapI = dict(qur={}, asc={}, lat={}, std={})
+
+        for u in sorted(Transcription.arabicTrans):
+            (qur, asc, lat, std) = Transcription.arabicTrans[u]
+            for (k, v) in zip(arabicMap.keys(), Transcription.arabicTrans[u]):
+                arabicMap[k].setdefault(v, u)
+                arabicMapI[k][u] = v
+
+        self.arabicMap = arabicMap
+        self.arabicMapI = arabicMapI
+
     def sycSplitPunc(self):
         pass
 
